/*****************************************************************************/
/*                                                                           */
/*   draw.cpp                                                                */
/*   www.xbdev.net                                                           */
/*                                                                           */
/*****************************************************************************/


#include <windows.h>
#include <assert.h>
#include <d3d9.h>
#include <d3dx9.h>

#include <stdio.h>  //sprintf(..)

#pragma comment(lib, "d3dx9.lib")
#pragma comment(lib, "d3d9.lib")




#define SZ_FX_FILE					"simple.fx"


extern LPDIRECT3DDEVICE9			g_pd3dDevice;
extern HWND							g_hWnd;

extern float						g_fSpinX;
extern float						g_fSpinY;
extern float						g_fZoom;


static ID3DXEffect*					g_pEffect				= NULL;
static LPDIRECT3DVERTEXDECLARATION9 g_pVertexDeclaration	= NULL;

static int							g_FPS					= 0;

static LPDIRECT3DVERTEXBUFFER9		g_pVertBuff				= NULL;

static float						g_fElpasedTime			= 0.0f;

// Forward Declarations

bool LoadShader();



struct CUSTOMVERTEX
{
	float p[3];
	float n[3];
	unsigned int c;
	float tu,tv;

	enum 
	{
		FVF_VERTEX = D3DFVF_XYZ|D3DFVF_NORMAL|D3DFVF_DIFFUSE|D3DFVF_TEX1
	};
};


CUSTOMVERTEX g_Vertices[] =
{
//     x     y     z      nx    ny    nz                         r    g    b    a        tu   tv    
    // Front Face
    {-1.0f, 1.0f,-1.0f,  0.0f, 0.0f,-1.0f,  D3DCOLOR_COLORVALUE(1.0f,1.0f,1.0f,1.0f),   0.0f,0.0f, },
    { 1.0f, 1.0f,-1.0f,  0.0f, 0.0f,-1.0f,  D3DCOLOR_COLORVALUE(1.0f,1.0f,1.0f,1.0f),   1.0f,0.0f, },
    {-1.0f,-1.0f,-1.0f,  0.0f, 0.0f,-1.0f,  D3DCOLOR_COLORVALUE(1.0f,1.0f,1.0f,1.0f),   0.0f,1.0f, },
    { 1.0f,-1.0f,-1.0f,  0.0f, 0.0f,-1.0f,  D3DCOLOR_COLORVALUE(1.0f,1.0f,1.0f,1.0f),   1.0f,1.0f, },
    // Back Face
    {-1.0f, 1.0f, 1.0f,  0.0f, 0.0f, 1.0f,  D3DCOLOR_COLORVALUE(1.0f,1.0f,1.0f,1.0f),   1.0f,0.0f, },
    {-1.0f,-1.0f, 1.0f,  0.0f, 0.0f, 1.0f,  D3DCOLOR_COLORVALUE(1.0f,1.0f,1.0f,1.0f),   1.0f,1.0f, },
    { 1.0f, 1.0f, 1.0f,  0.0f, 0.0f, 1.0f,  D3DCOLOR_COLORVALUE(1.0f,1.0f,1.0f,1.0f),   0.0f,0.0f, },
    { 1.0f,-1.0f, 1.0f,  0.0f, 0.0f, 1.0f,  D3DCOLOR_COLORVALUE(1.0f,1.0f,1.0f,1.0f),   0.0f,1.0f, },
    // Top Face
    {-1.0f, 1.0f, 1.0f,  0.0f, 1.0f, 0.0f,  D3DCOLOR_COLORVALUE(1.0f,1.0f,1.0f,1.0f),   0.0f,0.0f, },
    { 1.0f, 1.0f, 1.0f,  0.0f, 1.0f, 0.0f,  D3DCOLOR_COLORVALUE(1.0f,1.0f,1.0f,1.0f),   1.0f,0.0f, },
    {-1.0f, 1.0f,-1.0f,  0.0f, 1.0f, 0.0f,  D3DCOLOR_COLORVALUE(1.0f,1.0f,1.0f,1.0f),   0.0f,1.0f, },
    { 1.0f, 1.0f,-1.0f,  0.0f, 1.0f, 0.0f,  D3DCOLOR_COLORVALUE(1.0f,1.0f,1.0f,1.0f),   1.0f,1.0f, },
    // Bottom Face
    {-1.0f,-1.0f, 1.0f,  0.0f,-1.0f, 0.0f,  D3DCOLOR_COLORVALUE(1.0f,1.0f,1.0f,1.0f),   0.0f,1.0f, },
    {-1.0f,-1.0f,-1.0f,  0.0f,-1.0f, 0.0f,  D3DCOLOR_COLORVALUE(1.0f,1.0f,1.0f,1.0f),   0.0f,0.0f, },
    { 1.0f,-1.0f, 1.0f,  0.0f,-1.0f, 0.0f,  D3DCOLOR_COLORVALUE(1.0f,1.0f,1.0f,1.0f),   1.0f,1.0f, },
    { 1.0f,-1.0f,-1.0f,  0.0f,-1.0f, 0.0f,  D3DCOLOR_COLORVALUE(1.0f,1.0f,1.0f,1.0f),   1.0f,0.0f, },
    // Right Face
    { 1.0f, 1.0f,-1.0f,  1.0f, 0.0f, 0.0f,  D3DCOLOR_COLORVALUE(1.0f,1.0f,1.0f,1.0f),   0.0f,0.0f, },
    { 1.0f, 1.0f, 1.0f,  1.0f, 0.0f, 0.0f,  D3DCOLOR_COLORVALUE(1.0f,1.0f,1.0f,1.0f),   1.0f,0.0f, },
    { 1.0f,-1.0f,-1.0f,  1.0f, 0.0f, 0.0f,  D3DCOLOR_COLORVALUE(1.0f,1.0f,1.0f,1.0f),   0.0f,1.0f, },
    { 1.0f,-1.0f, 1.0f,  1.0f, 0.0f, 0.0f,  D3DCOLOR_COLORVALUE(1.0f,1.0f,1.0f,1.0f),   1.0f,1.0f, },
    // Left Face
    {-1.0f, 1.0f,-1.0f, -1.0f, 0.0f, 0.0f,  D3DCOLOR_COLORVALUE(1.0f,1.0f,1.0f,1.0f),   1.0f,0.0f, },
    {-1.0f,-1.0f,-1.0f, -1.0f, 0.0f, 0.0f,  D3DCOLOR_COLORVALUE(1.0f,1.0f,1.0f,1.0f),   1.0f,1.0f, },
    {-1.0f, 1.0f, 1.0f, -1.0f, 0.0f, 0.0f,  D3DCOLOR_COLORVALUE(1.0f,1.0f,1.0f,1.0f),   0.0f,0.0f, },
    {-1.0f,-1.0f, 1.0f, -1.0f, 0.0f, 0.0f,  D3DCOLOR_COLORVALUE(1.0f,1.0f,1.0f,1.0f),   0.0f,1.0f, }
};

const D3DVERTEXELEMENT9 declaration[] = 
{
	{ 0, 0  * sizeof(float),D3DDECLTYPE_FLOAT3,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 0 },  // pos
	{ 0, 3  * sizeof(float),D3DDECLTYPE_FLOAT3,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_NORMAL,   0 },  // normal
	{ 0, 6  * sizeof(float),D3DDECLTYPE_D3DCOLOR, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_COLOR,    0 },  // colour
	{ 0, 7  * sizeof(float),D3DDECLTYPE_FLOAT2,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 0 },  // tex0
	D3DDECL_END()
};




bool CreateShape()
{
	// Create vertex buffer
    CUSTOMVERTEX *pVertices;

    g_pd3dDevice->CreateVertexBuffer( sizeof(g_Vertices),
                                      D3DUSAGE_WRITEONLY, 0,
                                      D3DPOOL_DEFAULT, &g_pVertBuff, NULL );

    g_pVertBuff->Lock( 0, sizeof(g_Vertices), (void**)&pVertices, 0 );
    

	memcpy(pVertices, g_Vertices, sizeof(g_Vertices));

    g_pVertBuff->Unlock();

	return true;
}//End CreateShape(..)


bool Create()
{
	bool bOkay = true;
	
	bOkay &= CreateShape();

	bOkay &= LoadShader();

	return bOkay;
}//End Create(..)


bool Release()
{
	g_pVertBuff->Release();

	if( g_pEffect )
	{
		g_pEffect->Release();
		g_pVertexDeclaration->Release();
	}

	return true;
}// End Release()



bool LoadShader()
{
	LPD3DXBUFFER pBufferErrors;
	DWORD dwShaderFlags = 0;

	HRESULT hr = D3DXCreateEffectFromFile( g_pd3dDevice, SZ_FX_FILE,
		                      NULL, // CONST D3DXMACRO* pDefines
							  NULL, // LPD3DXINCLUDE pInclude
							  dwShaderFlags, NULL, &g_pEffect, &pBufferErrors);
	         
	if(FAILED(hr) )
	{
		LPVOID pCompileErrors = pBufferErrors->GetBufferPointer(); 
		MessageBox(NULL, (const char*)pCompileErrors, "Compile Error", 
		MB_OK|MB_ICONEXCLAMATION);

		return false;
	}

	g_pd3dDevice->CreateVertexDeclaration( declaration, &g_pVertexDeclaration );
	g_pd3dDevice->SetVertexDeclaration( g_pVertexDeclaration );

	return true;
}// End LoadShader(..)




void SetShaderConstants(float x, float y, float z)
// Setting Shader Constants
{		
	D3DXMATRIX matTrans;
	D3DXMATRIX matRot;
    D3DXMATRIX matProj,matIWorld,matView,matWorld;

	D3DXMatrixTranslation( &matTrans, x, y, z + g_fZoom ); 
	

	D3DXMatrixRotationYawPitchRoll( &matRot, D3DXToRadian(g_fSpinX), 
										     D3DXToRadian(g_fSpinY), 
												0.0f );
	
	D3DXMatrixPerspectiveFovLH( &matProj, D3DXToRadian( 45.0f ), 
                            640.0f / 480.0f, 0.1f, 500.0f );

	matWorld = matRot * matTrans;

	D3DXMatrixIdentity( &matView ); // This sample is not really making use of a view matrix

	D3DXMATRIX worldViewProj = matWorld * matView * matProj;

	D3DXMATRIX matWorldI = matWorld;
	D3DXMatrixTranspose(&matWorldI, &matWorldI);

	g_pEffect->SetMatrix("worldViewProj", &worldViewProj);

	g_pEffect->SetMatrix("matWorldI", &matWorldI);

}//End SetShaderConstants(..)



void UpdateTiming()
{
	float fCurTime     = (float)GetTickCount();
	static float fLastTime    = fCurTime;
	g_fElpasedTime = (float)((fCurTime - fLastTime) * 0.001);
	fLastTime    = fCurTime;

	// Simple performance checking Frames Per Second

	static float fSecondPassed = 0.0f;
	fSecondPassed += g_fElpasedTime;
	static int FPSCounter = 0;
	FPSCounter++;
	if( fSecondPassed > 1.0f )
	{
		fSecondPassed=0.0f;
		g_FPS=FPSCounter;
		FPSCounter=0;

		// Disply it to our window caption every second
		static char buf[200];
		sprintf(buf, "www.xbdev.net - FPS:%d", 
			          g_FPS);
		SetWindowText(g_hWnd, buf);
	}
	// End FPS
}//End UpdateTiming(..)


void RenderCube(float x, float y, float z)
{
	DWORD iPass = 0;
	UINT cPasses = 0;

	SetShaderConstants(x,y,z);

	g_pEffect->SetTechnique("Effect");
	g_pd3dDevice->SetVertexDeclaration( g_pVertexDeclaration );

	g_pd3dDevice->SetStreamSource( 0, g_pVertBuff, 0, sizeof(CUSTOMVERTEX) );

	g_pEffect->Begin(&cPasses, 0);	
	g_pEffect->BeginPass(0);

	g_pd3dDevice->DrawPrimitive( D3DPT_TRIANGLESTRIP,  0, 2 );
	g_pd3dDevice->DrawPrimitive( D3DPT_TRIANGLESTRIP,  4, 2 );
	g_pd3dDevice->DrawPrimitive( D3DPT_TRIANGLESTRIP,  8, 2 );
	g_pd3dDevice->DrawPrimitive( D3DPT_TRIANGLESTRIP, 12, 2 );
	g_pd3dDevice->DrawPrimitive( D3DPT_TRIANGLESTRIP, 16, 2 );
	g_pd3dDevice->DrawPrimitive( D3DPT_TRIANGLESTRIP, 20, 2 );

			
	//End of our Effect Render
	g_pEffect->EndPass();
	g_pEffect->End();
}// End RenderCube(..)



/***************************************************************************/
/*                                                                         */
/* render()                                                                */
/* Our main render loop, which gets called over and over agian to do our   */
/* drawing...clears the screen, draws the data, then presents it..         */
/*                                                                         */
/***************************************************************************/
bool Render()
{

	UpdateTiming();


#if(0)
	g_pd3dDevice->SetRenderState( D3DRS_FILLMODE, D3DFILL_WIREFRAME );
#else
	g_pd3dDevice->SetRenderState( D3DRS_FILLMODE, D3DFILL_SOLID );
#endif

	g_pd3dDevice->Clear( 0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, 0xffAA3300, 1.0f, 0 );
    g_pd3dDevice->BeginScene();
	
	
	RenderCube(-3,-3,0);
	RenderCube(-3, 0,0);
	RenderCube(-3, 3,0);

		
	RenderCube(0,-3,0);
	RenderCube(0, 0,0);
	RenderCube(0, 3,0);


	RenderCube(3,-3,0);
	RenderCube(3, 0,0);
	RenderCube(3, 3,0);

    g_pd3dDevice->EndScene();
	g_pd3dDevice->Present( NULL, NULL, NULL, NULL );


	return true;

}//End Render()
