/*****************************************************************************/
/*                                                                           */
/* bkenwright@xbdev.net                                                      */
/* www.xbdev.net                                                             */
/*                                                                           */
/*****************************************************************************/
/*

   
*/
/*****************************************************************************/


float4 vecLightDir = float4( 0.0f, 0.0f, 1.0f, 0.0f );


//---------------------------------------------------------------------------//

// transformations
float4x4 worldViewProj : WORLDVIEWPROJ;

float4x4 matWorldI : WORLD;

//------------------------------------
struct vertexInput {
    float3 position				: POSITION;
    float3 normal				: NORMAL;
    float4 colour               : COLOR;
    float2 tex0					: TEXCOORD0;
};

struct vertexOutput {
    float4 position	: POSITION; 
    float4 normal   : TEXCOORD0;   
    float4 colour   : COLOR;
    float2 tex0	    : TEXCOORD1; 
    
};



/******************************************************************************/
/*                                                                            */
/*   ***       ***  ********  ********   *********  ********  ***     ***     */
/*    ***     ***   ***       ***   ***  *********  ***        ***   ***      */
/*     ***   ***    ***       ***   ***     ***     ***         *******       */
/*      *** ***     ********  ***   ***     ***     ********     *****        */
/*       *****      ***       ********      ***     ***         *******       */
/*        ***       ***       ***  **       ***     ***        ***   ***      */
/*         *        ********  ***   ***     ***     ********  ***     ***     */
/*                                                                            */
/******************************************************************************/

float4 Lighting0(float4 vInColour, float3 vNormal)
{
	float4 normal = normalize(float4(vNormal,0));
    // Store normalized light vector
    float4 light = normalize( vecLightDir );

	float4 eyePosition = float4(0,0,0,0);
    // Calculate half angle vector     
    float4 eye = eyePosition;
    float4 halfvec = normalize( light + eye );                    
    
    // Calculate diffuse component
    float diffuse = dot( normal, light );

    // Calculate specular component
    float specular = dot( normal, halfvec );
    specular = pow( specular, 32 );

	// A little ambient color
	float4 ambientColor = float4( 0.2, 0.2, 0.2, 0.2 );

	// White specular
    float4 specularMaterial = float4( 0.5, 0.5, 0.5, 0.5 );

    // Combine diffuse and specular contributions and output final vertex color
    vInColour = diffuse * vInColour + specular * specularMaterial + ambientColor;
    
    return vInColour;
}// DirLighting(..)


//---------------------------------------------------------------------------//
vertexOutput VS(vertexInput IN) 
{
    vertexOutput OUT = (vertexOutput)0;
    
	float3 P = IN.position.xyz;

	OUT.tex0 = IN.tex0;
	
	
	float4 normal = -normalize( mul( matWorldI, IN.normal ).xyzz );
	
	//OUT.colour = Lighting0( IN.colour, normal ) ;
	
	OUT.colour = Lighting0( float4(1.0f, 1.0f, 0, 0), normal );
	
	OUT.position = mul(float4(P, 1.0f), worldViewProj);
    OUT.normal = normal;
    return OUT;
}




/******************************************************************************/
/*                                                                            */
/*            ********  ***   ****     ****    ******    ***                  */
/*            **    **  ***    ****   ****     ******    ***                  */
/*            **    **  ***      *** ***       ***       ***                  */
/*            ******    ***       *****        ******    ***                  */
/*            **        ***      *** ***       ***       ***                  */
/*            **        ***     ****  ****     ******    *********            */
/*            **        ***    ****    ****    ******    *********            */
/*                                                                            */
/******************************************************************************/


// Simple lighting function used in the pixel shader code
float4 DirLighting(float4 vInColour, float3 vNormal)
{
	float4 ambient = {0.2, 0.2, 0.2, 0.2};
	ambient = ambient * vInColour;
	float4 diffuse = vInColour;
	float4 light = normalize( vecLightDir );
	vInColour =ambient + diffuse * dot(vNormal, vecLightDir);
	return vInColour;	
}// DirLighting(..)



//---------------------------------------------------------------------------//
float4 PS( vertexOutput IN): COLOR
{
	float4 FinalColour = IN.colour;
	
	//FinalColour = DirLighting(FinalColour, IN.normal);
	
	return FinalColour;
}

//---------------------------------------------------------------------------//
technique Effect        
{   
    pass Surface
    {		
		VertexShader = compile vs_1_1 VS();
		PixelShader  = compile ps_1_3 PS();
        //--------------------------------------------------------------------//
		// Final = Dest * DestBlend + Src * SrcBlend, where dest is the pixel 
		// on the screen and src is the texel color.
		// blends in D3D are: SRCCOLOR, INVSRCCOLOR, SRCALPHA, INVSRCALPHA, 
		// DESTCOLOR, INVDESTCOLOR, DESTALPHA, INVDESTALPHA, ONE, and ZERO.

		// Final = Dest * DestBlend + Src * SrcBlend
		
		//SrcBlend = SRCALPHA;
		//DestBlend = INVSRCALPHA;
		//AlphaBlendEnable = true;
		
		AlphaBlendEnable = false;
		SrcBlend = SRCCOLOR;
		DestBlend = DESTCOLOR;
		ZEnable=true;
		ZWriteEnable = true;
		CullMode = CCW;
		
		//BlendOp = Add;
		//Lighting=false;
		//CullMode = None;
    }
    
}// End technique Fur

